#ifndef __ASM_SH_MOBILE_LCDC_H__
#define __ASM_SH_MOBILE_LCDC_H__

#include <linux/fb.h>

/* Header Section */
struct lcdrt_sectioninfo {
	u32 variableareaaddr;	/* Address of Variable Area */
	u32 variableareasize;	/* Size of Variable Area */
	u32 fixedareaaddr;	/* Address of Fixed Area */
	u32 fixedareasize;	/* Size of Fixed Area */
	u32 commandareaaddr;	/* Address of Command Transfer Area */
	u32 commandareasize;	/* Size of Command Transfer Area */
	u32 onscrnaddr;		/* Address of OnScreen Buffer */
	u32 onscrnsize;		/* Size of OnScreen Buffer */
};

enum { RGB8,   /* 24bpp, 8:8:8 */
       RGB9,   /* 18bpp, 9:9 */
       RGB12A, /* 24bpp, 12:12 */
       RGB12B, /* 12bpp */
       RGB16,  /* 16bpp */
       RGB18,  /* 18bpp */
       RGB24,  /* 24bpp */
       SYS8A,  /* 24bpp, 8:8:8 */
       SYS8B,  /* 18bpp, 8:8:2 */
       SYS8C,  /* 18bpp, 2:8:8 */
       SYS8D,  /* 16bpp, 8:8 */
       SYS9,   /* 18bpp, 9:9 */
       SYS12,  /* 24bpp, 12:12 */
       SYS16A, /* 16bpp */
       SYS16B, /* 18bpp, 16:2 */
       SYS16C, /* 18bpp, 2:16 */
       SYS18,  /* 18bpp */
       SYS24 };/* 24bpp */

enum { LCDC_CHAN_DISABLED = 0,
       LCDC_CHAN_MAINLCD,
       LCDC_CHAN_SUBLCD };

enum { LCDC_CLK_BUS, LCDC_CLK_PERIPHERAL, LCDC_CLK_EXTERNAL,
       LCDC_CLK_HDMI, LCDC_CLK_DV };

#define LCDC_FLAGS_DWPOL (1 << 0) /* Rising edge dot clock data latch */
#define LCDC_FLAGS_DIPOL (1 << 1) /* Active low display enable polarity */
#define LCDC_FLAGS_DAPOL (1 << 2) /* Active low display data polarity */
#define LCDC_FLAGS_HSCNT (1 << 3) /* Disable HSYNC during VBLANK */
#define LCDC_FLAGS_DWCNT (1 << 4) /* Disable dotclock during blanking */

#define SH_FB_DISPLAY_LAYER1 0
#define SH_FB_DISPLAY_LAYER2 1
#define SH_FB_DISPLAY_LAYER3 2
#define SH_FB_DISPLAY_LAYER4 3

#define SH_FB_DISPLAY_RGB565   0
#define SH_FB_DISPLAY_RGB888   1
#define SH_FB_DISPLAY_ARGB8888 2
#define SH_FB_DISPLAY_RGBA5551 3

#define SH_FB_LAY_NOBUSY 0
#define SH_FB_LAY_BUSY   1

#define SH_FB_DOUBLE_BUFF 1

#define SH_FB_BLEND_LCD 0
#define SH_FB_OVERLAY 1

#define SH_FB_DISP_OFF 0

#define SH_FB_SDENC_HW_START 0
#define SH_FB_SDENC_HW_STOP  1
#define SH_FB_SDENC_DISP_ON  0
#define SH_FB_SDENC_DISP_OFF 1

/*Main display*/
#define SH_MLCD_WIDTH		800
#define SH_MLCD_HEIGHT		480

#define SH_MLCD_TRCOLOR		0
#define SH_MLCD_REPLACECOLOR	0
#define SH_MLCD_RECTX		0
#define SH_MLCD_RECTY		0

/*Sub display*/
#define SH_SLCD_WIDTH		720
#define SH_SLCD_HEIGHT		480

#define SH_SLCD_TRCOLOR		0
#define SH_SLCD_REPLACECOLOR	0
#define SH_SLCD_RECTX		0
#define SH_SLCD_RECTY		0

#define FBIO_WAITFORVSYNC _IOW('F', 0x20, __u32)

struct sh_mobile_lcdc_lcd_size_cfg { /* width and height of panel in mm */
	unsigned long width;
	unsigned long height;
};

enum {
	SH_FIXUP_NONE = 0,
	SH_FIXUP_RGB565,
	SH_FIXUP_RGB888,
	SH_FIXUP_XRGB8888,
	SH_FIXUP_YUV420,
	SH_FIXUP_YUV422,
	SH_FIXUP_YUV444,
	SH_FIXUP_END,
};

struct sh_videomode_ext {
	unsigned long flags; /* LCDC_FLAGS_... */
	int (*fixup)(struct fb_info *info, int fixup_num);
};

struct sh_mobile_lcdc_chan_cfg {
	int chan;
	int bpp;
	int interface_type; /* selects RGBn or SYSn I/F, see above */
	int clock_divider;
	unsigned long flags; /* LCDC_FLAGS_... */
	struct fb_videomode lcd_cfg;
	struct sh_mobile_lcdc_lcd_size_cfg lcd_size_cfg;

	struct fb_videomode *modedb;
	struct sh_videomode_ext *modedb_ext;
	unsigned int modedb_size;
	const char *default_mode;
};

struct sh_mobile_lcdc_info {
	int clock_source;
	struct sh_mobile_lcdc_chan_cfg ch[2];
};

struct fb_lay_supportpixfmt {
	int level;
	int pixfmt;
};

struct fb_lay_open {
	int level;
	int buffernum;
	int overlay;
};

struct fb_lay_close {
	int level;
};

struct fb_lay_disp {
	int level;
	int on;
};

struct fb_lay_alpha {
	int level;
	unsigned short alpha;
};

struct fb_lay_keycolor {
	int level;
	long keycolor;
};

struct fb_lay_data_size {
	int level;
	unsigned int x;
	unsigned int y;
};

struct fb_lay_pixfmt {
	int level;
	unsigned short pixfmt;
};

struct fb_lay_pos {
	int level;
	unsigned int x;
	unsigned int y;
};

struct fb_lay_display_size {
	int level;
	unsigned int x;
	unsigned int y;
};

struct fb_viewlay {
	int level;
	unsigned int data_x;
	unsigned int data_y;
	unsigned int pos_x;
	unsigned int pos_y;
	unsigned int display_x;
	unsigned int display_y;
};

struct fb_swapaddr {
	int level;
	unsigned char *swapaddr;
	unsigned int datasize;
	unsigned int app_id;
	unsigned int offset;
};

struct fb_waitswap {
	int level;
};

struct fb_lay_busy {
	int level;
	unsigned int laybusy;
};

#define IOC_SH_MOBILE_FB_MAGIC 'S'
#define FB_IOC_ISLAYEX \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x00, __u32)
#define FB_IOC_SUPPORTPIXFMT \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x01, struct fb_lay_supportpixfmt)
#define FB_IOCS_OPEN \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x02, struct fb_lay_open)
#define FB_IOCS_CLOSE \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x03, struct fb_lay_close)
#define FB_IOCS_DISP \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x04, struct fb_lay_disp)
#define FB_IOCS_ALPHA \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x05, struct fb_lay_alpha)
#define FB_IOCG_ALPHA \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x06, struct fb_lay_alpha)
#define FB_IOCS_KEYCOLOR \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x07, struct fb_lay_keycolor)
#define FB_IOCG_KEYCOLOR \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x08, struct fb_lay_keycolor)
#define FB_IOCS_LAYSIZE \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x09, struct fb_lay_data_size)
#define FB_IOCG_LAYSIZE \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0A, struct fb_lay_data_size)
#define FB_IOCS_PIXFMT \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0B, struct fb_lay_pixfmt)
#define FB_IOCG_PIXFMT \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0C, struct fb_lay_pixfmt)
#define FB_IOCS_POS \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0D, struct fb_lay_pos)
#define FB_IOCG_POS \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0E, struct fb_lay_pos)
#define FB_IOCS_SIZE \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x0F, struct fb_lay_display_size)
#define FB_IOCG_SIZE \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x10, struct fb_lay_display_size)
#define FB_IOCS_VIEWLAY \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x11, struct fb_viewlay)
#define FB_IOCS_SWAPADDR \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x12, struct fb_swapaddr)
#define FB_IOC_WAITSWAP \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x13, struct fb_waitswap)
#define FB_IOCG_LAYBUSY \
	_IOW(IOC_SH_MOBILE_FB_MAGIC, 0x14, struct fb_lay_busy)



extern int sh_mobile_lcdc_keyclr_set(unsigned short s_key_clr,
				     unsigned short output_mode);
extern int sh_mobile_lcdc_alpha_set(unsigned short s_alpha,
				     unsigned short output_mode);
extern int sh_mobile_lcdc_refresh(unsigned short set_state,
				     unsigned short output_mode);

extern struct semaphore   sh_mobile_sem_hdmi;

#endif /* __ASM_SH_MOBILE_LCDC_H__ */
