/*
 *  File Name       : RSPF0_spu_api_dataport.h
 *  Function        : SPU OPEN SOUND I/F
 *  Release Version : Ver 1.00
 *  Release Date    : 2011/11/30
 *
 *  Copyright (C) 2011 Renesas Electronics Corporation
 *
 *  This program is free software;you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by Free
 *  Softwere Foundation; either version 2 of License, or (at your option) any
 *  later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program;
 *  If not, write to the Free Software Foundation, Inc., 51 Franklin St,
 *  Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#ifndef __RSPF0_SPU_API_DATAPORT_H__
#define __RSPF0_SPU_API_DATAPORT_H__

enum {
	SPUDATAPORT_DIRECTION_IN = 0, /* CPU -> DSP */
	SPUDATAPORT_DIRECTION_OUT,    /* DSP -> CPU */
};

enum {
	SPUDATAPORT_TRANSFER_MODE_SINGLE1 = 0x0,
	SPUDATAPORT_TRANSFER_MODE_SINGLE2 = 0x1,
	SPUDATAPORT_TRANSFER_MODE_SINGLE3 = 0x2,
	SPUDATAPORT_TRANSFER_MODE_SINGLE4 = 0x3,
	SPUDATAPORT_TRANSFER_MODE_BURST_ALIGNMENT1 = 0x4,
	SPUDATAPORT_TRANSFER_MODE_BURST_ALIGNMENT2 = 0x5,
	SPUDATAPORT_TRANSFER_MODE_BURST_ALIGNMENT3 = 0x6,
	SPUDATAPORT_TRANSFER_MODE_BURST_SEAMLESS = 0x7,
};

enum {
	SPUDATAPORT_SWAP_MODE_NONE = 0x0,
	SPUDATAPORT_SWAP_MODE_BYTE,
	SPUDATAPORT_SWAP_MODE_WORD,
	SPUDATAPORT_SWAP_MODE_BYTE_WORD,
};

enum {
	SPUDATAPORT_DATAUNIT_WORD = 0x0,
	SPUDATAPORT_DATAUNIT_BYTE = 0x1,
	SPUDATAPORT_DATAUNIT_HALFWORD = 0x2,
};

enum {
	SPUDATAPORT_DMA_CH0 = 0x0,
	SPUDATAPORT_DMA_CH1,
	SPUDATAPORT_DMA_CH2,
};

enum {
	SPUDATAPORT_DMABUF_0 = 0x0,
	SPUDATAPORT_DMABUF_1,
	SPUDATAPORT_DMABUF_2,
	SPUDATAPORT_DMABUF_3,
};

enum {
	SPUDATAPORT_INTERCONNECT_OFF = 0x0,
	SPUDATAPORT_INTERCONNECT_ON,
};

enum {
	SPUDATAPORT_END_CONDITION_DMA_END = 0x0,
	SPUDATAPORT_END_CONDITION_BUFFER_AVAILABLE,
};

enum {
	SPUDATAPORT_INITIAL_THRESHOLD_DEFAULT = 0x0,
	SPUDATAPORT_INITIAL_THRESHOLD_HALF,
	SPUDATAPORT_INITIAL_THRESHOLD_FULL,
};

#define SPUDATAPORT_NOT_FLUSHED  (0)
#define SPUDATAPORT_FLUSHED      (1)

typedef struct {
	unsigned long data_unit;
	unsigned long direction;
	unsigned long dma_ch;
	unsigned long dma_transfer_mode;
	unsigned long dma_swap_mode;
	unsigned long end_notify_condition;
} sSPUDataPortParam;

typedef struct {
	unsigned long data_unit;
	unsigned long direction;
	unsigned long dsp_id;
	unsigned long dmabuf_id;
	unsigned long dma_ch;
	unsigned long initial_threshold_mode;
} sSPUDataPortInterconnectParam;

#if 0 /* Move to RSPF0_API.h */
typedef struct {
	unsigned long port_id;
	unsigned long dsp_id;
	unsigned long dma_transfer_mode;
	unsigned long data_unit;
} sSPUDataPort;
#endif

typedef void (*tSPUDataPortTransferEndCallBack)
		(sSPUDataPort *port, unsigned long size, unsigned long flush);

extern long SPUDataPort_Init(unsigned long dsp_id);

extern long SPUDataPort_SetCallBack(sSPUDataPort *port,
					tSPUDataPortTransferEndCallBack end_cb);

extern long SPUDataPort_Create(unsigned long dsp_id, sSPUDataPort *port);

extern long SPUDataPort_Delete(sSPUDataPort *port);

extern long SPUDataPort_SetParam(sSPUDataPort *port, sSPUDataPortParam *param,
					sSPUBuffer *buffer, unsigned long sync);

extern long SPUDataPort_Enable(sSPUDataPort *port, unsigned long sync);

extern long SPUDataPort_Disable(sSPUDataPort *port, unsigned long *rest_size,
							unsigned long sync);

extern long SPUDataPort_TransferStart(sSPUDataPort *port, unsigned long addr,
							unsigned long size);

extern long SPUDataPort_GetTransferCount(sSPUDataPort *port,
						unsigned long *rest_size);

#endif /* __RSPF0_SPU_API_DATAPORT_H__ */

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("SPU OPEN SOUND I/F Ver 1.0");
MODULE_AUTHOR("Renesas Electronics");
