/*
 *  File Name       : RSPF0_spu_api_system.c
 *  Function        : SPU OPEN SOUND I/F
 *  Release Version : Ver 1.00
 *  Release Date    : 2011/11/30
 *
 *  Copyright (C) 2011 Renesas Electronics Corporation
 *
 *  This program is free software;you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by Free
 *  Softwere Foundation; either version 2 of License, or (at your option) any
 *  later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A
 *  PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program;
 *  If not, write to the Free Software Foundation, Inc., 51 Franklin St,
 *  Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#include "RSPF0_spu_api.h"
#include "RSPF0_spu_api_system.h"

#define CLASS_ID 0x01

/* sign extention macro */
#define INT24_TO_INT(x)             \
	(((x) & 0x00800000) ?       \
	((x) | 0xff000000) :  (x))

enum {
	API_SYSTEM_GET_VERSION_REQUEST         = SPUAPI_API_ID(CLASS_ID, 0x00),
	API_SYSTEM_GET_VERSION_RESPONSE        = SPUAPI_API_ID(CLASS_ID, 0x80),
	API_SYSTEM_GET_MODULE_LIST_REQUEST     = SPUAPI_API_ID(CLASS_ID, 0x01),
	API_SYSTEM_GET_MODULE_LIST_RESPONSE    = SPUAPI_API_ID(CLASS_ID, 0x81),
	API_SYSTEM_GET_MODULE_VERSION_REQUEST  = SPUAPI_API_ID(CLASS_ID, 0x02),
	API_SYSTEM_GET_MODULE_VERSION_RESPONSE = SPUAPI_API_ID(CLASS_ID, 0x82),
	API_SYSTEM_CREATE_BUFFER_REQUEST       = SPUAPI_API_ID(CLASS_ID, 0x03),
	API_SYSTEM_CREATE_BUFFER_RESPONSE      = SPUAPI_API_ID(CLASS_ID, 0x83),
	API_SYSTEM_DELETE_BUFFER_REQUEST       = SPUAPI_API_ID(CLASS_ID, 0x04),
	API_SYSTEM_DELETE_BUFFER_RESPONSE      = SPUAPI_API_ID(CLASS_ID, 0x84),
	API_SYSTEM_RESET_BUFFER_REQUEST        = SPUAPI_API_ID(CLASS_ID, 0x05),
	API_SYSTEM_RESET_BUFFER_RESPONSE       = SPUAPI_API_ID(CLASS_ID, 0x85),
	API_SYSTEM_GET_BUFFER_STATUS_REQUEST   = SPUAPI_API_ID(CLASS_ID, 0x06),
	API_SYSTEM_GET_BUFFER_STATUS_RESPONSE  = SPUAPI_API_ID(CLASS_ID, 0x86),
	API_SYSTEM_GET_MEMORY_STATUS_REQUEST   = SPUAPI_API_ID(CLASS_ID, 0x07),
	API_SYSTEM_GET_MEMORY_STATUS_RESPONSE  = SPUAPI_API_ID(CLASS_ID, 0x87),
	API_SYSTEM_FLUSH_BUFFER_REQUEST        = SPUAPI_API_ID(CLASS_ID, 0x08),
	API_SYSTEM_FLUSH_BUFFER_RESPONSE       = SPUAPI_API_ID(CLASS_ID, 0x88),
	API_SYSTEM_INSTALL_MODULE_REQUEST      = SPUAPI_API_ID(CLASS_ID, 0x09),
	API_SYSTEM_INSTALL_MODULE_RESPONSE     = SPUAPI_API_ID(CLASS_ID, 0x89),
	API_SYSTEM_UNINSTALL_MODULE_REQUEST    = SPUAPI_API_ID(CLASS_ID, 0x0a),
	API_SYSTEM_UNINSTALL_MODULE_RESPONSE   = SPUAPI_API_ID(CLASS_ID, 0x8a),
	API_SYSTEM_IO_WRITE_REQUEST            = SPUAPI_API_ID(CLASS_ID, 0x0b),
	API_SYSTEM_IO_WRITE_RESPONSE           = SPUAPI_API_ID(CLASS_ID, 0x8b),
	API_SYSTEM_IO_READ_REQUEST             = SPUAPI_API_ID(CLASS_ID, 0x0c),
	API_SYSTEM_IO_READ_RESPONSE            = SPUAPI_API_ID(CLASS_ID, 0x8c),
	API_SYSTEM_LOG_CTRL_REQUEST            = SPUAPI_API_ID(CLASS_ID, 0x0d),
	API_SYSTEM_LOG_CTRL_RESPONSE           = SPUAPI_API_ID(CLASS_ID, 0x8d),
};

static unsigned long response[SPUAPI_DSP_MAX][SPUAPI_MESSAGE_LENGTH];

#define SYSTEM_PARAM_MASK  0x00FFFFFF

long SPUSystem_GetVersion(unsigned long dsp_id, unsigned long *firmware_version,
					unsigned long *framework_version) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;

	if (dsp_id >= SPUAPI_DSP_MAX || !firmware_version || !framework_version)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_GET_VERSION_REQUEST;
	size         = 0;
	response_api = API_SYSTEM_GET_VERSION_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, 0, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	*firmware_version = SYSTEM_PARAM_MASK & response[dsp_id][0];
	*framework_version = SYSTEM_PARAM_MASK & response[dsp_id][1];

	return 0;
}

static long SPUSystem_DefaultRequest(unsigned long dsp_id, unsigned long api_id,
	unsigned long *data, unsigned long size, unsigned long response_api,
							unsigned long sync) {
	if (sync == SPUAPI_NO_RESPONSE) {
		response_api = 0;
		api_id = SPUAPI_NO_RESPONSE_API(api_id);
	}

	if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	if (sync == SPUAPI_NO_RESPONSE)
		return 0;
	else
		return response[dsp_id][0];
}

/* API for Buffer */
long SPUSystem_CreateBuffer(unsigned long dsp_id, unsigned long buf_size,
				unsigned long type, sSPUBuffer *buffer) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[2];
	unsigned long result;

	if (dsp_id >= SPUAPI_DSP_MAX || !buffer ||
	type > SPUAPI_BUFFER_TYPE_BYTE || buf_size == 0)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_CREATE_BUFFER_REQUEST;
	size         = 2;
	data[0]      = buf_size;
	data[1]      = type;
	response_api = API_SYSTEM_CREATE_BUFFER_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	result = (long)response[dsp_id][0];
	if (result == 0) {
		buffer->dsp_id    = dsp_id;
		buffer->buffer_id = response[dsp_id][1];
		buffer->size      = buf_size;
	}
	return result;
}


long SPUSystem_DeleteBuffer(sSPUBuffer *buffer)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!buffer)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_DELETE_BUFFER_REQUEST;
	size         = 1;
	data[0]      = buffer->buffer_id;
	response_api = API_SYSTEM_DELETE_BUFFER_RESPONSE;

	if (SPUAPI_SendApi(buffer->dsp_id, api_id, data, size, response_api) <
	0)
		return SPUAPI_ERROR_DRIVER;

	return response[buffer->dsp_id][0];
}

long SPUSystem_ResetBuffer(sSPUBuffer *buffer, unsigned long sync)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!buffer)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_RESET_BUFFER_REQUEST;
	size         = 1;
	data[0]      = buffer->buffer_id;
	response_api = API_SYSTEM_RESET_BUFFER_RESPONSE;

	return SPUSystem_DefaultRequest(buffer->dsp_id, api_id, data, size,
		response_api, sync);
}

long SPUSystem_FlushBuffer(sSPUBuffer *buffer, unsigned long sync)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!buffer)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_FLUSH_BUFFER_REQUEST;
	size         = 1;
	data[0]      = buffer->buffer_id;
	response_api = API_SYSTEM_FLUSH_BUFFER_RESPONSE;

	return SPUSystem_DefaultRequest(buffer->dsp_id, api_id, data, size,
		response_api, sync);
}

/* API for Debug */

static unsigned long module_list[SPUAPI_DSP_MAX][32];

int SPUAPI_SystemGetModuleList(unsigned long dsp_id, unsigned long **list,
							unsigned long *num) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long i;
	unsigned long count;
	unsigned long page;
	unsigned long data[1];

	if (dsp_id >= SPUAPI_DSP_MAX || !list || !num)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_GET_MODULE_LIST_REQUEST;
	size         = 1;
	response_api = API_SYSTEM_GET_MODULE_LIST_RESPONSE;

	page = 0;
	count = 0;

	while (1) {
		data[0] = page;
		if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) <
		0)
			return SPUAPI_ERROR_DRIVER;

		for (i = 0; i < 16; i++) {
			if (response[dsp_id][i] != 0) {
				module_list[dsp_id][count] =
							response[dsp_id][i];
				count++;
			} else
				break;
		}
		if (i != 16)
			break;
		else
			page++;
	}
	*list = module_list[dsp_id];
	*num = count;

	return 0;
}

long SPUSystem_GetModuleVersion(unsigned long dsp_id, unsigned long module_id,
							unsigned long *ver) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (dsp_id >= SPUAPI_DSP_MAX || !ver)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_GET_MODULE_VERSION_REQUEST;
	size         = 1;
	data[0]      = module_id;
	response_api = API_SYSTEM_GET_MODULE_VERSION_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;
	*ver = SYSTEM_PARAM_MASK & response[dsp_id][2];

	return response[dsp_id][0];
}

long SPUSystem_GetBufferStatus(sSPUBuffer *buffer, sSPUBufferStatus *status)
{
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (!buffer || !status)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_GET_BUFFER_STATUS_REQUEST;
	size         = 1;
	data[0]      = buffer->buffer_id;
	response_api = API_SYSTEM_GET_BUFFER_STATUS_RESPONSE;

	if (SPUAPI_SendApi(buffer->dsp_id, api_id, data, size, response_api) <
	0)
		return SPUAPI_ERROR_DRIVER;

	status->count = SYSTEM_PARAM_MASK & response[buffer->dsp_id][2];
	status->size  = SYSTEM_PARAM_MASK & response[buffer->dsp_id][3];
	status->addr  = SYSTEM_PARAM_MASK & response[buffer->dsp_id][4];
	status->rp    = SYSTEM_PARAM_MASK & response[buffer->dsp_id][5];
	status->wp    = SYSTEM_PARAM_MASK & response[buffer->dsp_id][6];
	return response[buffer->dsp_id][0];
}

long SPUSystem_GetMemoryStatus(unsigned long dsp_id, unsigned long memory_id,
						sSPUMemoryStatus *status) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (dsp_id >= SPUAPI_DSP_MAX || !status)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_GET_MEMORY_STATUS_REQUEST;
	size         = 1;
	data[0]      = memory_id;
	response_api = API_SYSTEM_GET_MEMORY_STATUS_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;

	status->start_addr    = SYSTEM_PARAM_MASK & response[dsp_id][1];
	status->size          = SYSTEM_PARAM_MASK & response[dsp_id][2];
	status->error         = SYSTEM_PARAM_MASK & response[dsp_id][3];
	status->heap.free     = SYSTEM_PARAM_MASK & response[dsp_id][4];
	status->heap.used     = SYSTEM_PARAM_MASK & response[dsp_id][5];
	status->heap.max      = SYSTEM_PARAM_MASK & response[dsp_id][6];
	status->heap.block    = SYSTEM_PARAM_MASK & response[dsp_id][7];
	status->aligned.free  = SYSTEM_PARAM_MASK & response[dsp_id][8];
	status->aligned.used  = SYSTEM_PARAM_MASK & response[dsp_id][9];
	status->aligned.block = SYSTEM_PARAM_MASK & response[dsp_id][10];

	return response[dsp_id][0];
}

/* API for IO */
long SPUSystem_IoWrite(unsigned long dsp_id, unsigned long addr,
				unsigned long reg_value, unsigned long sync) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[2];

	if (dsp_id >= SPUAPI_DSP_MAX || (addr & 0x000C0000) != 0x000C0000)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_IO_WRITE_REQUEST;
	size         = 2;
	data[0]      = (addr >> 2) & 0xffff;
	data[1]      = reg_value;
	response_api = API_SYSTEM_IO_WRITE_RESPONSE;

	if (SPUSystem_DefaultRequest(dsp_id, api_id, data, size, response_api,
	sync) < 0)
		return SPUAPI_ERROR_DRIVER;

	return 0;
}

long SPUSystem_IoRead(unsigned long dsp_id, unsigned long addr,
						unsigned long *reg_value) {
	unsigned long api_id;
	unsigned long size;
	unsigned long response_api;
	unsigned long data[1];

	if (dsp_id >= SPUAPI_DSP_MAX || (addr & 0x000C0000) != 0x000C0000)
		return SPUAPI_ERROR_PARAM;

	api_id       = API_SYSTEM_IO_READ_REQUEST;
	size         = 1;
	data[0]      = (addr >> 2) & 0xffff;
	response_api = API_SYSTEM_IO_READ_RESPONSE;

	if (SPUAPI_SendApi(dsp_id, api_id, data, size, response_api) < 0)
		return SPUAPI_ERROR_DRIVER;
	*reg_value = SYSTEM_PARAM_MASK & response[dsp_id][0];
	return 0;
}

static void SPUSystem_ApiReceiveHandler(unsigned long dsp_id,
	unsigned long api_id, unsigned long *data, unsigned long size) {
	unsigned long i;

	for (i = 0; i < size; i++)
		response[dsp_id][i] = INT24_TO_INT(data[i]);	/* pgr0788 */
}

long SPUSystem_Init(unsigned long dsp_id)
{
	if (SPUAPI_RegisterApiReceiveHandler(dsp_id, CLASS_ID,
	SPUSystem_ApiReceiveHandler) != 0)
		return -1;
	return 0;
}

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("SPU OPEN SOUND I/F Ver 1.0");
MODULE_AUTHOR("Renesas Electronics");
